package edu.nyu.scps.where;

import android.content.Context;
import android.graphics.PointF;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;


public class MainActivity extends ActionBarActivity implements SensorEventListener {
    private SensorManager sensorManager;	//Can't initialize them here.
    private Sensor sensor;

    private boolean first = true;
    private long firstTime;
    private long lastTime;

    private PointF currentLocation = new PointF();
    private PointF lastVelocity = new PointF();
    private PointF lastAcceleration = new PointF();

    private TextView textView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        textView = (TextView)findViewById(R.id.textView);

        sensorManager = (SensorManager)getSystemService(Context.SENSOR_SERVICE);
        sensor = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (sensor != null) {
            sensorManager.registerListener(this, sensor, SensorManager.SENSOR_DELAY_NORMAL);
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (sensor != null) {
            sensorManager.unregisterListener(this);
        }
    }

    @Override
    public void onSensorChanged(SensorEvent sensorEvent) {
        long currentTime = sensorEvent.timestamp;
        PointF currentAcceleration = new PointF(
                sensorEvent.values[0],
                sensorEvent.values[1]
        );

        if (first) {
            //Can't compute the interval the first time this method is called,
            //because there is no value for lastTime.
            first = false;
        } else {
            float interval = (currentTime - lastTime) / 1000000000f; //seconds
            float elapsed = (currentTime - firstTime) / 1000000000f; //since start

            PointF currentVelocity = new PointF(
                    lastVelocity.x + interval * (lastAcceleration.x + currentAcceleration.x) / 2f,
                    lastVelocity.y + interval * (lastAcceleration.y + currentAcceleration.y) / 2f
            );

            currentLocation.offset(
                    interval * (lastVelocity.x + currentVelocity.x) / 2f,
                    interval * (lastVelocity.y + currentVelocity.y) / 2f
            );

            String s = "x = " + currentLocation.x + "\n"
                    + "y = " + currentLocation.y + "\n"
                    + "vx = " + currentVelocity.x + "\n"
                    + "vy = " + currentVelocity.y + "\n"
                    + "ax = " + currentAcceleration.x + "\n"
                    + "ay = " + currentAcceleration.y + "\n"
                    + "interval = " + interval + "\n"
                    + "elapsed = " + elapsed;

            textView.setText(s);
            lastVelocity.set(currentVelocity);
        }

        lastAcceleration.set(currentAcceleration);
        lastTime = currentTime;
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
